/***************************************************************************
 *   Copyright 2009-2011 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
***************************************************************************/

#ifndef PALAPELI_COLLECTION_P_H
#define PALAPELI_COLLECTION_P_H

#include "collection.h"

class Palapeli::Collection::Item : public QObject, public QStandardItem
{
	Q_OBJECT
	public:
		Item(Palapeli::Puzzle* puzzle);

		Palapeli::Puzzle* puzzle() const { return m_puzzle; }
	public Q_SLOTS:
		void populate();
	private:
		Palapeli::Puzzle* m_puzzle;
};

#endif // PALAPELI_COLLECTION_P_H
